#include "MyModel.h"
#include "mat.h"
#define _USE_MATH_DEFINES // for M_PI
#include <cmath>



MyModel::MyModel(const char* name) :
	Model(name),
	spinBase("Spin base", 0, 360, 180, 1),
	udShoulder1("Raise arm 1", 60, 180, 90, 1),
	udShoulder2("Raise arm 2", 0, 120, 90, 1),
	ioElbow1("Elbow 1 in/out", 60, 270, 90, 1),
	ioElbow2("Elbow 2 in/out", 0, 210, 180, 1),
	pinchers1("Pinchers 1", 60, 140, 90, 1),
	pinchers2("Pinchers 2", 60, 140, 90, 1),
	wrist1("Wrist 1", 0, 360, 0, 1),
	wrist2("Wrist 2", 0, 360, 0, 1),
	pincherSize("Pincher Size", .5, 1.5, 1, 0.1)
{
	properties.add(&spinBase);
	properties.add(&udShoulder1);
	properties.add(&udShoulder2);
	properties.add(&ioElbow1);
	properties.add(&ioElbow2);
	properties.add(&pinchers1);
	properties.add(&pinchers2);
	properties.add(&wrist1);
	properties.add(&wrist2);
	properties.add(&pincherSize);
}



void MyModel::draw() {
	Vec4<int> v;
	Mat4<int> m;
	auto v2 = v * m;
	v2 = m * v;



	ModelerDrawState *mds = ModelerDrawState::Instance();
	int divisions;
	switch (mds->m_quality)
	{
	case HIGH:
		divisions = 128; break;
	case MEDIUM:
		divisions = 32; break;
	case LOW:
		divisions = 16; break;
	case POOR:
		divisions = 8; break;
	}


	/* remember which matrix mode OpenGL was in. */
	int savemode;
	glGetIntegerv(GL_MATRIX_MODE, &savemode);

	/* switch to the model matrix and scale by x,y,z. */
	glMatrixMode(GL_MODELVIEW);

	//body
	glRotatef(spinBase.getValue(), 0, 1, 0);
	glTranslatef(0, 1, 0);
	drawSphere(3, divisions);
	glPushMatrix(); //head
	{
		glTranslatef(-1.25f, 2.5f, -1.25f);
		drawCube(2.5, 2.5, 2.5);
		glPopMatrix();
	}
	glPushMatrix();//arm 1
	{
		glTranslatef(2.5, 1.5, 0);
		glRotatef(udShoulder1.getValue() - 90, 0, 0, 1);
		drawSphere(1, divisions);//shoulder
		glPushMatrix(); {
			glTranslatef(0, -.2, -.3);
			drawCube(.6, 3, .6);//upper arm
			glPushMatrix(); {
				glTranslatef(3, .3, .3);
				drawSphere(0.5, divisions);//elbow
				glPushMatrix(); {
					glRotatef(ioElbow1.getValue() - 90, 0, 0, 1);
					glTranslatef(-.3, 0, -.3);
					drawCube(.6, .6, -2);//forearm
					glPushMatrix(); {

						glTranslatef(.3, -2, .3);
						glRotatef(wrist1.getValue(), 0, 1, 0);
						drawSphere(0.5, divisions);//wrist
						glScalef(pincherSize.getValue(), pincherSize.getValue(), 1);
						glPushMatrix(); { //finger1
							glRotatef(pinchers1.getValue()-90, 0, 0, 1);
							glTranslatef(0, -.15, -.15);
							drawCube(.3, -1.25, .3);
							glTranslatef(-1.25, 0, 0);
							drawCube(.3, .3, -.7);
							glPopMatrix();
						}
						glPushMatrix(); { //finger2
							glRotatef(-(pinchers1.getValue()-90), 0, 0, 1);
							glTranslatef(0, -.15, -.15);
							drawCube(.3, 1.25, .3);
							glTranslatef(.95, 0, 0);
							drawCube(.3, .3, -.7);
							glPopMatrix();
						}
						glPopMatrix();
					}
					glPopMatrix();
				}
				glPopMatrix();
			}

			glPopMatrix();
		}

		glPopMatrix();
	}

	glPushMatrix();//arm 2 
	{
		glTranslatef(-2.5, 1.5, 0);
		glRotatef(udShoulder2.getValue() - 90, 0, 0, 1);
		drawSphere(1, divisions);//shoulder
		glPushMatrix(); {
			glTranslatef(0, -.2, -.3);
			drawCube(.6, -3, .6); // upper arm
			glPushMatrix(); {
				glTranslatef(-3, .3, .3);
				drawSphere(0.5, divisions); // elbow
				glPushMatrix(); {
					glRotatef(ioElbow2.getValue() - 180, 0, 0, 1);
					glTranslatef(-.3, 0, -.3);
					drawCube(.6, .6, -2); // forearm
					glPushMatrix(); {
						glTranslatef(.3, -2, .3);

						glRotatef(wrist2.getValue(), 0, 1, 0);
						drawSphere(0.5, divisions); //wrist
						glScalef(pincherSize.getValue(), pincherSize.getValue(), 1);
						glPushMatrix(); { //finger1

							glRotatef(pinchers2.getValue() - 90, 0, 0, 1);
							glTranslatef(0, -.15, -.15);
							drawCube(.3, -1.25, .3);
							glTranslatef(-1.25, 0, 0);
							drawCube(.3, .3, -.7);
							glPopMatrix();
						}
						glPushMatrix(); { //finger2
							glRotatef(-(pinchers2.getValue() - 90), 0, 0, 1);
							glTranslatef(0, -.15, -.15);
							drawCube(.3, 1.25, .3);
							glTranslatef(.95, 0, 0);
							drawCube(.3, .3, -.7);
							glPopMatrix();
						}


						glPopMatrix();
					}
					glPopMatrix();
				}

				glPopMatrix();
			}
			glPopMatrix();
		}
		glPopMatrix();
	}

	/* restore the model matrix stack, and switch back to the matrix
	mode we were in. */

	glMatrixMode(savemode);

}

void MyModel::drawCube(float l, float w, float h)
{

	//glScaled(5, 5, 5);

	glBegin(GL_QUADS);

	////Back
	glNormal3d(0.0, 0.0, -1.0);
	glVertex3d(0.0, 0.0, 0.0);
	glVertex3d(0.0, h, 0.0);
	glVertex3d(w, h, 0.0);
	glVertex3d(w, 0.0, 0.0);

	////Bottom
	glNormal3d(0.0, -1.0, 0.0);
	glVertex3d(0.0, 0.0, 0.0);
	glVertex3d(w, 0.0, 0.0);
	glVertex3d(w, 0.0, l);
	glVertex3d(0.0, 0.0, l);

	////Left
	glNormal3d(-1.0, 0.0, 0.0);
	glVertex3d(0.0, 0.0, 0.0);
	glVertex3d(0.0, 0.0, l);
	glVertex3d(0.0, h, l);
	glVertex3d(0.0, h, 0.0);


	////Front
	glNormal3d(0.0, 0.0, 1.0);
	glVertex3d(0.0, 0.0, l);
	glVertex3d(w, 0.0, l);
	glVertex3d(w, h, l);
	glVertex3d(0.0, h, l);

	////Top
	glNormal3d(0.0, h, 0.0);
	glVertex3d(0.0, h, 0.0);
	glVertex3d(0.0, h, l);
	glVertex3d(w, h, l);
	glVertex3d(w, h, 0.0);


	////Right
	glNormal3d(1.0, 0.0, 0.0);
	glVertex3d(w, 0.0, 0.0);
	glVertex3d(w, h, 0.0);
	glVertex3d(w, h, l);
	glVertex3d(w, 0.0, l);

	glEnd();

}

void MyModel::drawSphere(float r, int divisions) {

	int n = divisions;

	for (int i = 0; i < n / 2; i++) {
		float a1 = i * (M_PI * 2) / n - (M_PI / 2);
		float a2 = (i + 1) * (M_PI * 2) / n - (M_PI / 2);
		glBegin(GL_QUAD_STRIP);
		for (int j = 0; j <= n; j++) {
			float a3 = j * (M_PI * 2) / n;
			float n1_x = cos(a2) * sin(a3);
			float n1_y = cos(a2) * cos(a3);
			float n1_z = sin(a2);

			float p1_x = (float)(r * n1_x);
			float p1_y = (float)(r * n1_y);
			float p1_z = (float)(r * n1_z);
			glNormal3f(n1_x, n1_y, n1_z);
			glTexCoord2f((GLfloat)(1.0 - j / (float)n), (GLfloat)(2 * (i + 1) / (float)n));
			glVertex3f(p1_x, p1_y, p1_z);

			n1_x = cos(a1) * sin(a3);
			n1_y = cos(a1) * cos(a3);
			n1_z = sin(a1);

			p1_x = (float)(r * n1_x);
			p1_y = (float)(r * n1_y);
			p1_z = (float)(r * n1_z);
			glNormal3f(n1_x, n1_y, n1_z);
			glTexCoord2f((GLfloat)(1.0 - j / (float)n), (GLfloat)(2 * i / (float)n));
			glVertex3f(p1_x, p1_y, p1_z);
		}
		glEnd();
	}
}

void MyModel::drawCylinder(double h, double r1, double r2, int divisions) {
	GLUquadricObj* gluq;

	/* GLU will again do the work.  draw the sides of the cylinder. */
	gluq = gluNewQuadric();
	gluQuadricDrawStyle(gluq, GLU_FILL);
	gluQuadricTexture(gluq, GL_FALSE);
	gluCylinder(gluq, r1, r2, h, divisions, divisions);
	gluDeleteQuadric(gluq);

	if (r1 > 0.0)
	{
		/* if the r1 end does not come to a point, draw a flat disk to
		cover it up. */

		gluq = gluNewQuadric();
		gluQuadricDrawStyle(gluq, GLU_FILL);
		gluQuadricTexture(gluq, GL_TRUE);
		gluQuadricOrientation(gluq, GLU_INSIDE);
		gluDisk(gluq, 0.0, r1, divisions, divisions);
		gluDeleteQuadric(gluq);
	}

	if (r2 > 0.0)
	{
		/* if the r2 end does not come to a point, draw a flat disk to
		cover it up. */

		glTranslated(0.0, 0.0, h);

		/* draw a disk centered at the new origin. */
		gluq = gluNewQuadric();
		gluQuadricDrawStyle(gluq, GLU_FILL);
		gluQuadricTexture(gluq, GL_TRUE);
		gluQuadricOrientation(gluq, GLU_OUTSIDE);
		gluDisk(gluq, 0.0, r2, divisions, divisions);
		gluDeleteQuadric(gluq);
	}
}
