#pragma once
#pragma warning (disable : 4305)
#pragma warning (disable : 4244)
#pragma warning (disable : 4786)
#pragma warning (disable : 4312)

#include "vault.h"
#include "FL/glut.h"
#include "model.h"
#include "MyModel.h"
#include <cmath>
#include <time.h>

/** A scene, which includes the lights and models. */
class Scene : public Model {
protected:
	Texture2D texture;
	ShaderProgram shader;
	ShaderProgram spotShader;
	ShaderProgram toonShader;

	// Scene lights
	PointLight pointLight;
	DirectionalLight directionalLight;
	SpotLight spotLight;

	// Switches for spheres
	BooleanProperty useTexture;
	// Some slider properties
	RangeProperty rotateX, rotateY;
	RangeProperty brightness;
	RangeProperty toonLine;
	// Diffuse color picker
	RGBProperty diffuse;
	// Shader chooser
	ChoiceProperty shaderChoice;

	// Sample model
	MyModel sample;

	// Switch for showing/hiding reference unit sphere
	BooleanProperty showReferenceUnitSphere;

	// Lets you pick what shapes to use for the default model!
	ChoiceProperty shapeChoice;



public:
	/** Construct the scene */
	Scene(const char* name);

	/**
	* Modeler calls this method once an OpenGL context becomes available,
	* (which happens before the first time draw() is called), and whenever
	* you click the "Reload Textures and Shaders" menu option.
	*
	* Make sure you call the load() methods of each of your textures and
	* shaders here.
	*/
	void load();

	/** Draw the scene. */
	void draw();

	/** draw() calls this method to draw the 3d objects in the scene. */
	void drawModel();

	/**
	* Modeler calls this method many times a second when the Animate
	* checkbox is checked.
	*/
	void tick();

	/** SwapTexture changes the default checker texture used by scene.cpp to the one specified. */
	void swapTexture(const char* filename);


};